
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity carousel is
    generic (
        C_HIGHADDR   : std_logic_vector(0 to 31) := X"00000000";
        C_BASEADDR   : std_logic_vector(0 to 31) := X"FFFFFFFF";
        C_LMB_AWIDTH : integer                   := 32;
        C_LMB_DWIDTH : integer                   := 32
    ) ;
    port (
        LMB_Clk                     : in std_logic := '0';
        LMB_Rst                     : in std_logic := '0';

        LMB_ABus                    : in  std_logic_vector(0 to 31);
        LMB_WriteDBus               : in  std_logic_vector(0 to 31);
        LMB_AddrStrobe              : in  std_logic;
        LMB_ReadStrobe              : in  std_logic;
        LMB_WriteStrobe             : in  std_logic;
        LMB_BE                      : in  std_logic_vector(0 to 3);
        Sl_DBus                     : out std_logic_vector(0 to 31);
        Sl_Ready                    : out std_logic;

        BRAM_Rst_A                  : out std_logic;
        BRAM_Clk_A                  : out std_logic;
        BRAM_Addr_A                 : out std_logic_vector(0 to 31);
        BRAM_EN_A                   : out std_logic;
        BRAM_WEN_A                  : out std_logic_vector(0 to 3);
        BRAM_Dout_A                 : out std_logic_vector(0 to 31);
        BRAM_Din_A                  : in  std_logic_vector(0 to 31);

		FSL_Clk	                    : in  std_logic;
		FSL_Rst	                    : in  std_logic;

		CPU_FSL_OUT_Clk	            : in  std_logic;
		CPU_FSL_OUT_Read	        : in  std_logic;
		CPU_FSL_OUT_Data	        : out std_logic_vector(0 to 31);
		CPU_FSL_OUT_Control	        : out std_logic;
		CPU_FSL_OUT_Exists	        : out std_logic;

		CPU_FSL_IN_Clk	            : in  std_logic;
		CPU_FSL_IN_Write	        : in  std_logic;
		CPU_FSL_IN_Data	            : in  std_logic_vector(0 to 31);
		CPU_FSL_IN_Control	        : in  std_logic;
		CPU_FSL_IN_Full	            : out std_logic;

        CAROUSEL_FSL_IN_Clk         : out std_logic;
        CAROUSEL_FSL_IN_Read        : out std_logic;
        CAROUSEL_FSL_IN_Data        : in  std_logic_vector(0 to 31);
        CAROUSEL_FSL_IN_Control     : in  std_logic;
        CAROUSEL_FSL_IN_Exists      : in  std_logic;

        CAROUSEL_FSL_OUT_Clk        : out std_logic;
        CAROUSEL_FSL_OUT_Write      : out std_logic;
        CAROUSEL_FSL_OUT_Data       : out std_logic_vector(0 to 31);
        CAROUSEL_FSL_OUT_Control    : out std_logic;
        CAROUSEL_FSL_OUT_Full       : in  std_logic
	);

attribute SIGIS : string; 
attribute SIGIS of FSL_Clk : signal is "Clk"; 

end carousel;


architecture SPM of carousel is

   signal cpu_spm_out               : std_logic_vector(31 downto 0);
   signal lmb_select_0, lmb_select_1: std_logic;
   signal spm_0, lmb_select_d       : std_logic;

begin

    -- Address decoder
    lmb_select_0 <= '1' when LMB_ABus(0 to 3) = "0000" else '0';
    lmb_select_1 <= '1' when LMB_ABus(0 to 3) = "0001" else '0';

    -- Non-DMA (boot) SPM
    BRAM_Rst_A  <= '0';
    BRAM_Clk_A  <= LMB_Clk;
    BRAM_EN_A   <= '1';
    BRAM_WEN_A  <= LMB_BE 
                when (LMB_WriteStrobe and 
                        LMB_AddrStrobe and lmb_select_0) = '1' else "0000";
    BRAM_Dout_A <= LMB_WriteDBus;
    Sl_DBus     <= BRAM_Din_A when spm_0 = '1' else cpu_spm_out;
    BRAM_Addr_A <= LMB_ABus;

    Ready_Handling : process (LMB_Clk, LMB_Rst) is
    begin
        if (LMB_Rst = '1') then
            Sl_Ready <= '0';
            spm_0 <= '1';
            lmb_select_d <= '0';
        elsif (LMB_Clk'event and LMB_Clk = '1') then  -- rising clock edge
            Sl_Ready <= LMB_AddrStrobe and (lmb_select_1 or lmb_select_0);
            if (lmb_select_1 and LMB_AddrStrobe) = '1' then
                spm_0 <= '0';
            elsif (lmb_select_0 and LMB_AddrStrobe) = '1' then
                spm_0 <= '1';
            end if;
            lmb_select_d <= lmb_select_0;
        end if;
    end process Ready_Handling;

    CAROUSEL_FSL_IN_Clk <= FSL_Clk;
    CAROUSEL_FSL_OUT_Clk <= FSL_Clk;

    c : entity logic
        port map (
            clk => FSL_Clk,
            rst => FSL_Rst,

            fsl_to_cpu_read => CPU_FSL_OUT_Read,
            fsl_to_cpu_data => CPU_FSL_OUT_Data,
            fsl_to_cpu_control => CPU_FSL_OUT_Control,
            fsl_to_cpu_exists => CPU_FSL_OUT_Exists,

            fsl_from_cpu_write => CPU_FSL_IN_Write,
            fsl_from_cpu_data => CPU_FSL_IN_Data,
            fsl_from_cpu_control => CPU_FSL_IN_Control,
            fsl_from_cpu_full => CPU_FSL_IN_Full,

            fsl_from_mem_read => CAROUSEL_FSL_IN_Read,
            fsl_from_mem_data => CAROUSEL_FSL_IN_Data,
            fsl_from_mem_control => CAROUSEL_FSL_IN_Control,
            fsl_from_mem_exists => CAROUSEL_FSL_IN_Exists,

            fsl_to_mem_write => CAROUSEL_FSL_OUT_Write,
            fsl_to_mem_data => CAROUSEL_FSL_OUT_Data,
            fsl_to_mem_control => CAROUSEL_FSL_OUT_Control,
            fsl_to_mem_full => CAROUSEL_FSL_OUT_Full,

            spm_addr1 => LMB_ABus,
            spm_addr2 => LMB_ABus,
            spm_to_cpu1 => cpu_spm_out);

end architecture SPM;


